"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CourseService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const course_schema_1 = require("../schemas/course.schema");
const user_service_1 = require("./user.service");
const lesson_service_1 = require("./lesson.service");
const cloudinary_service_1 = require("./cloudinary.service");
const stream_1 = require("stream");
const courseRoutine_service_1 = require("./courseRoutine.service");
const enrollments_service_1 = require("./enrollments.service");
const section_service_1 = require("./section.service");
let CourseService = class CourseService {
    constructor(courseModel, userService, courseRoutineService, lessonService, cloudinaryService, enrollmentsService, sectionService) {
        this.courseModel = courseModel;
        this.userService = userService;
        this.courseRoutineService = courseRoutineService;
        this.lessonService = lessonService;
        this.cloudinaryService = cloudinaryService;
        this.enrollmentsService = enrollmentsService;
        this.sectionService = sectionService;
    }
    async create(course) {
        try {
            await this.checkCourseNameExists(course.name);
            if (course.thumbnail) {
                const thumbnailStream = new stream_1.Readable();
                thumbnailStream.push(Buffer.from(course.thumbnail, "base64"));
                thumbnailStream.push(null);
                const thumbnailUrl = await this.cloudinaryService.uploadImage(thumbnailStream);
                course.thumbnail = thumbnailUrl;
            }
            const createdCourse = new this.courseModel(course);
            return createdCourse.save();
        }
        catch (error) {
            throw error;
        }
    }
    async getAllCourses() {
        return this.courseModel.find().exec();
    }
    async update(courseId, updatedCourse) {
        var _a;
        await this.checkCourseNameExists(updatedCourse.name, courseId);
        if (updatedCourse.thumbnail &&
            !((_a = updatedCourse.thumbnail) === null || _a === void 0 ? void 0 : _a.startsWith("https"))) {
            const thumbnailStream = new stream_1.Readable();
            thumbnailStream.push(Buffer.from(updatedCourse.thumbnail, "base64"));
            thumbnailStream.push(null);
            const thumbnailUrl = await this.cloudinaryService.uploadImage(thumbnailStream);
            if (updatedCourse.thumbnail) {
                updatedCourse.thumbnail = thumbnailUrl;
            }
        }
        const existingCourse = await this.courseModel.findByIdAndUpdate(courseId, updatedCourse, { new: true });
        if (!existingCourse) {
            throw new common_1.NotFoundException(`Course with ID ${courseId} not found`);
        }
        return existingCourse;
    }
    async getCourseById(courseId, studentId) {
        var _a, _b;
        const course = await this.courseModel.findById(courseId).exec();
        if (!course) {
            throw new common_1.NotFoundException(`Course with ID ${courseId} not found`);
        }
        let enrollments;
        if (studentId) {
            enrollments = await this.enrollmentsService.getByStudentAndCourseId(courseId, studentId);
        }
        const multiInstructor = (course === null || course === void 0 ? void 0 : course.multiInstructorIds)
            ? await Promise.all((_a = JSON.parse(course === null || course === void 0 ? void 0 : course.multiInstructorIds)) === null || _a === void 0 ? void 0 : _a.map(async (item) => {
                return this.userService.getById(item === null || item === void 0 ? void 0 : item.value);
            }))
            : null;
        const instructorInfo = await this.userService.getById(course === null || course === void 0 ? void 0 : course.instructorId);
        const routineInfo = await this.courseRoutineService.getByCourseId(course === null || course === void 0 ? void 0 : course._id) || null;
        const sections = await this.sectionService.getByCourseId(course === null || course === void 0 ? void 0 : course._id) || null;
        const firstLesson = await this.lessonService.getBySectionId((_b = sections[0]) === null || _b === void 0 ? void 0 : _b._id) || null;
        return {
            course,
            lesson: firstLesson[0],
            courseInstructor: (multiInstructor === null || multiInstructor === void 0 ? void 0 : multiInstructor.length) > 0 ? multiInstructor : [instructorInfo],
            courseRoutine: routineInfo || null,
            hasRoutines: !!routineInfo,
            hasMultiInstructors: !!multiInstructor,
            isAuthUser: (course === null || course === void 0 ? void 0 : course.isFree) ? true : (enrollments === null || enrollments === void 0 ? void 0 : enrollments.isAmountPaid) ? true : false
        };
    }
    async getCoursesByOrganizationId(organizationId) {
        const courses = await this.courseModel.find({ organizationId }).exec();
        if (!courses || courses.length === 0) {
            throw new common_1.NotFoundException(`No courses found for organization with ID ${organizationId}`);
        }
        return courses;
    }
    async getCoursesByStatusAndType(status, courseType) {
        const query = {};
        if (status) {
            query.status = status;
        }
        if (courseType) {
            query.courseType = courseType;
        }
        const courses = await this.courseModel.find(query).exec();
        if (!courses || courses.length === 0) {
            throw new common_1.NotFoundException(`No courses found for ${status ? `status: ${status}` : ""}${status && courseType ? " and " : ""}${courseType ? `courseType: ${courseType}` : ""}`);
        }
        return courses;
    }
    async getCoursesByInstructorId(instructorId) {
        const courses = await this.courseModel
            .find({ instructor: instructorId })
            .exec();
        if (!courses || courses.length === 0) {
            throw new common_1.NotFoundException(`No courses found for instructor with ID ${instructorId}`);
        }
        return courses;
    }
    async getCoursesByType(courseType) {
        const courses = await this.courseModel
            .find({ courseType: courseType })
            .exec();
        if (!courses || courses.length === 0) {
            throw new common_1.NotFoundException(`No courses found for instructor with ID ${courseType}`);
        }
        return courses;
    }
    async delete(courseId) {
        const deletedCourse = await this.courseModel
            .findByIdAndDelete(courseId)
            .exec();
        if (!deletedCourse) {
            throw new common_1.NotFoundException(`Course with ID ${courseId} not found`);
        }
    }
    async checkCourseNameExists(name, courseId) {
        const existingCourse = await this.courseModel.findOne({
            name,
            _id: { $ne: courseId },
        });
        if (existingCourse) {
            throw new Error(`Course is '${name}' already exists`);
        }
        return existingCourse;
    }
    async getTotalCourses(courseType) {
        const totalCourses = await this.courseModel.countDocuments({ courseType });
        return totalCourses;
    }
    async getTotalActiveCourses(courseType) {
        const totalActiveCourses = await this.courseModel.countDocuments({ courseType, status: 'Approved' });
        return totalActiveCourses;
    }
    async getTotalPendingCourses(courseType) {
        const totalPendingCourses = await this.courseModel.countDocuments({ courseType, status: 'Pending' });
        return totalPendingCourses;
    }
    async getTotalFreeCourses(courseType) {
        const totalPendingCourses = await this.courseModel.countDocuments({ courseType, isFree: true });
        return totalPendingCourses;
    }
    async getTotalPaidCourses(courseType) {
        const totalPendingCourses = await this.courseModel.countDocuments({ courseType, isFree: false });
        return totalPendingCourses;
    }
};
CourseService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectModel)(course_schema_1.Course.name)),
    __metadata("design:paramtypes", [mongoose_2.Model,
        user_service_1.UserService,
        courseRoutine_service_1.CourseRoutineService,
        lesson_service_1.LessonService,
        cloudinary_service_1.CloudinaryService,
        enrollments_service_1.EnrollmentsService,
        section_service_1.SectionService])
], CourseService);
exports.CourseService = CourseService;
//# sourceMappingURL=course.service.js.map