"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PaymentService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const payment_schema_1 = require("../schemas/payment.schema");
let PaymentService = class PaymentService {
    constructor(paymentModel) {
        this.paymentModel = paymentModel;
    }
    async create(payment) {
        const createdPayment = new this.paymentModel(payment);
        return createdPayment.save();
    }
    async edit(paymentId, updatedPayment) {
        const existingPayment = await this.paymentModel
            .findByIdAndUpdate(paymentId, updatedPayment, { new: true })
            .exec();
        if (!existingPayment) {
            throw new common_1.NotFoundException(`Payment with ID ${paymentId} not found`);
        }
        return existingPayment;
    }
    async update(updatedPayment) {
        const { enrollmentsId } = updatedPayment, paymentDetails = __rest(updatedPayment, ["enrollmentsId"]);
        const existingPayment = await this.paymentModel
            .findOneAndUpdate({ enrollmentsId }, paymentDetails, { new: true })
            .exec();
        if (!existingPayment) {
            throw new common_1.NotFoundException(`Payment with Enrollment ID ${enrollmentsId} not found`);
        }
    }
    async getById(paymentId) {
        const payment = await this.paymentModel.findById(paymentId).exec();
        if (!payment) {
            throw new common_1.NotFoundException(`Payment with ID ${paymentId} not found`);
        }
        return payment;
    }
    async getByStudentId(studentId) {
        return this.paymentModel.find({ studentId }).exec();
    }
    async getByCourseId(courseId) {
        return this.paymentModel.find({ courseId }).exec();
    }
    async getByCourseIdAndStudentId(courseId, studentId) {
        return this.paymentModel.find({ courseId, studentId }).exec();
    }
    async getByEnrollmentsId(enrollmentsId) {
        return this.paymentModel.find({ enrollmentsId }).exec();
    }
    async delete(paymentId) {
        await this.getById(paymentId);
        await this.paymentModel.findByIdAndDelete(paymentId).exec();
    }
    async deleteByEnrollmentId(enrollmentId) {
        await this.paymentModel.deleteMany({ enrollmentsId: enrollmentId }).exec();
    }
};
PaymentService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectModel)(payment_schema_1.Payment.name)),
    __metadata("design:paramtypes", [mongoose_2.Model])
], PaymentService);
exports.PaymentService = PaymentService;
//# sourceMappingURL=payment.service.js.map