"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.QuestionService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const question_schema_1 = require("../schemas/question.schema");
let QuestionService = class QuestionService {
    constructor(questionModel) {
        this.questionModel = questionModel;
    }
    async create(questions) {
        const createdQuestions = [];
        for (const question of questions) {
            await this.checkQuestionTextNotExists(question.questionName, question.lessonId);
            const createdQuestion = new this.questionModel(question);
            createdQuestions.push(await createdQuestion.save());
        }
        return createdQuestions;
    }
    async edit(questions) {
        const updatedQuestions = [];
        for (const { questionId, updatedQuestion } of questions) {
            await this.checkQuestionTextNotExists(updatedQuestion.questionName, updatedQuestion.lessonId, questionId);
            const existingQuestion = await this.questionModel.findByIdAndUpdate(questionId, updatedQuestion, {
                new: true
            }).exec();
            if (!existingQuestion) {
                throw new common_1.NotFoundException(`Question with ID ${questionId} not found`);
            }
            updatedQuestions.push(existingQuestion);
        }
        return updatedQuestions;
    }
    async getAll() {
        return this.questionModel.find().exec();
    }
    async getById(questionId) {
        const question = await this.questionModel.findById(questionId).exec();
        if (!question) {
            throw new common_1.NotFoundException(`Question with id ${questionId} not found`);
        }
        return question;
    }
    async getByLessonId(lessonId) {
        return this.questionModel.find({ lessonId }).exec();
    }
    async delete(questionId) {
        await this.getById(questionId);
        await this.questionModel.findByIdAndDelete(questionId).exec();
    }
    async deleteByLessonId(lessonId) {
        await this.questionModel.deleteMany({ lessonId }).exec();
    }
    async checkQuestionTextNotExists(text, lessonId, excludedQuestionId) {
        const existingQuestion = await this.questionModel
            .findOne({ questionName: text, lessonId, _id: { $ne: excludedQuestionId } })
            .exec();
        if (existingQuestion) {
            throw new common_1.ConflictException(`Question "${text}" already exists for the given lesson`);
        }
    }
};
QuestionService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectModel)(question_schema_1.Question.name)),
    __metadata("design:paramtypes", [mongoose_2.Model])
], QuestionService);
exports.QuestionService = QuestionService;
//# sourceMappingURL=question.service.js.map