"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const bcrypt = require("bcrypt");
const jwt = require("jsonwebtoken");
const mongoose_2 = require("mongoose");
const nodemailer = require("nodemailer");
const user_schema_1 = require("../schemas/user.schema");
const stream_1 = require("stream");
const cloudinary_service_1 = require("./cloudinary.service");
let UserService = class UserService {
    constructor(userModel, cloudinaryService) {
        this.userModel = userModel;
        this.cloudinaryService = cloudinaryService;
    }
    async findAll() {
        const users = await this.userModel.find().exec();
        return users.map((user) => this.mapUserToDTO(user));
    }
    async findByRoleAndStatus(role, status) {
        const query = {};
        if (role) {
            query.role = role;
        }
        if (status !== undefined) {
            query.status = status;
        }
        return this.userModel.find(query).exec();
    }
    async create(user) {
        const hashedPassword = await bcrypt.hash(user.password, 10);
        if (user.image) {
            const thumbnailStream = new stream_1.Readable();
            thumbnailStream.push(Buffer.from(user.image, "base64"));
            thumbnailStream.push(null);
            const thumbnailUrl = await this.cloudinaryService.uploadImage(thumbnailStream);
            user.image = thumbnailUrl;
        }
        const createUser = new this.userModel(Object.assign(Object.assign({}, user), { password: hashedPassword }));
        return createUser.save();
    }
    async findUser(phone, username) {
        return this.userModel.findOne({ $or: [{ phone }, { username }] }).exec();
    }
    async getById(id) {
        const user = await this.userModel.findOne({ _id: id }).exec();
        return this.mapUserToDTO(user);
    }
    async edit(id, updatedUser) {
        var _a;
        if (updatedUser.image &&
            !((_a = updatedUser.image) === null || _a === void 0 ? void 0 : _a.startsWith("https"))) {
            const thumbnailStream = new stream_1.Readable();
            thumbnailStream.push(Buffer.from(updatedUser.image, "base64"));
            thumbnailStream.push(null);
            const thumbnailUrl = await this.cloudinaryService.uploadImage(thumbnailStream);
            if (updatedUser.image) {
                updatedUser.image = thumbnailUrl;
            }
        }
        const existingUser = await this.userModel
            .findByIdAndUpdate(id, updatedUser, { new: true })
            .exec();
        if (!existingUser) {
            throw new common_1.NotFoundException(`User with ID ${id} not found`);
        }
        return existingUser;
    }
    async delete(id) {
        const result = await this.userModel.findByIdAndDelete(id).exec();
        if (!result) {
            throw new common_1.NotFoundException(`User with ID ${id} not found`);
        }
    }
    async activateUser(id, status) {
        const existingUser = await this.userModel
            .findByIdAndUpdate(id, { status: status }, { new: true })
            .exec();
        if (!existingUser) {
            throw new common_1.NotFoundException(`User with ID ${id} not found`);
        }
        return existingUser;
    }
    async resetPassword(id, newPassword) {
        const user = await this.userModel.findOne({ _id: id }).exec();
        if (!user) {
            throw new common_1.NotFoundException(`User with Id ${id} not found`);
        }
        const hashedPassword = await bcrypt.hash(newPassword, 10);
        await this.userModel
            .findByIdAndUpdate(id, { password: hashedPassword }, { new: true })
            .exec();
        return user;
    }
    async loginUser(email, password, phone) {
        let foundUser;
        if (email) {
            foundUser = await this.userModel.findOne({ email }).exec();
        }
        else {
            foundUser = await this.userModel.findOne({ phone }).exec();
        }
        if (!foundUser) {
            throw new common_1.NotFoundException("User not found");
        }
        if (email) {
            const isMatchPassword = await bcrypt.compare(password, foundUser === null || foundUser === void 0 ? void 0 : foundUser.password);
            if (!isMatchPassword) {
                throw new common_1.NotFoundException("Invalid credentials");
            }
        }
        const accessToken = jwt.sign({ UserInfo: { email: foundUser === null || foundUser === void 0 ? void 0 : foundUser.email, role: foundUser === null || foundUser === void 0 ? void 0 : foundUser.role, id: foundUser._id } }, process.env.ACCESS_TOKEN_SECRET, { expiresIn: '10m' });
        const refreshToken = jwt.sign({ UserInfo: { email: foundUser === null || foundUser === void 0 ? void 0 : foundUser.email, role: foundUser === null || foundUser === void 0 ? void 0 : foundUser.role, id: foundUser._id } }, process.env.REFRESH_TOKEN_SECRET, { expiresIn: '90d' });
        foundUser.refreshToken = refreshToken;
        await foundUser.save();
        return { accessToken, refreshToken, user: foundUser };
    }
    async refreshAccessToken(refreshToken) {
        const loggedInUser = await this.userModel.findOne({ refreshToken }).exec();
        if (!loggedInUser) {
            throw new common_1.NotFoundException("Invalid refresh token");
        }
        const accessToken = jwt.sign({ UserInfo: { email: loggedInUser.email, role: loggedInUser.role, id: loggedInUser._id } }, process.env.ACCESS_TOKEN_SECRET, { expiresIn: '15m' });
        return accessToken;
    }
    async getMe(id) {
        const loggedInUser = await this.userModel.findOne({ _id: id }).exec();
        if (!loggedInUser) {
            throw new common_1.NotFoundException(`User with Id ${id} not found`);
        }
        return loggedInUser;
    }
    async logoutUser(refreshToken) {
        const loggedInUser = await this.userModel.findOne({ refreshToken }).exec();
        if (loggedInUser) {
            loggedInUser.refreshToken = "";
            await loggedInUser.save();
        }
    }
    mapUserToDTO(user) {
        return {
            id: user.id,
            name: user.name,
            username: user.username,
            email: user.email,
            phone: user.phone,
            organizationId: user.organizationId,
            role: user.role,
            status: user.status,
            info: user.info,
            address: user.address,
            image: user.image,
        };
    }
    async sendPasswordResetEmail(email) {
        const transporter = nodemailer.createTransport({
            service: "gmail",
            host: "smtp.gmail.com",
            port: 465,
            secure: true,
            auth: {
                user: "iqbaldetective@gmail.com",
                pass: "01836420972",
            },
        });
        const mailOptions = {
            from: "iqbaldetective@gmail.com",
            to: email,
            subject: "Password Reset",
            text: "Your password has been reset successfully.",
        };
        await transporter.sendMail(mailOptions);
    }
};
UserService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectModel)(user_schema_1.User.name)),
    __metadata("design:paramtypes", [mongoose_2.Model,
        cloudinary_service_1.CloudinaryService])
], UserService);
exports.UserService = UserService;
//# sourceMappingURL=user.service.js.map